import React, { Suspense, useEffect, useMemo, useState } from 'react';
import {
  Button,
  Input,
  VISUALIZATION_TYPES,
  Visualizations,
  useAppDispatch,
  useGraphQueryResultMeta,
  useVisualization,
  useActiveVisualization,
} from '../../..';
import { SettingsHeader } from './components';
import {
  removeVisualization,
  updateActiveVisualization,
  updateVisualization,
  updateActiveVisualizationAttributes,
} from '@graphpolaris/shared/lib/data-access/store/visualizationSlice';
import { VisualizationSettingsPropTypes, VisualizationSettingsType } from '../../common';

type Props = {};

export function VisualizationSettings({}: Props) {
  // const manager = VisualizationManager();
  // const activeVisualization = useActiveVisualization();
  const { activeVisualizationIndex, openVisualizationArray } = useVisualization();
  const activeVisualization = useMemo(
    () => openVisualizationArray[activeVisualizationIndex],
    [activeVisualizationIndex, openVisualizationArray],
  );
  const graphMetadata = useGraphQueryResultMeta();

  const dispatch = useAppDispatch();

  const [component, setComponent] = useState<
    | undefined
    | {
        component: React.FC<VisualizationSettingsPropTypes>;
        id: string;
      }
  >(undefined);

  useEffect(() => {
    loadVisualization(activeVisualization);
  }, [activeVisualization]);

  const loadVisualization = async (vis?: VisualizationSettingsType) => {
    if (!vis) {
      setComponent(undefined);
      return;
    }

    const componentModule = await Visualizations[vis.id]();
    const component = componentModule.default;

    setComponent({
      component: component.settingsComponent,
      id: vis.id,
    });
  };

  const updateSettings = (newSettings: Record<string, any>) => {
    if (activeVisualization) {
      dispatch(updateActiveVisualizationAttributes(newSettings));
    }
  };

  return (
    <div className="flex flex-col w-full overflow-auto">
      <div className="text-sm px-4 py-2 flex flex-col gap-1">
        <div className="flex justify-between items-center">
          <span className="font-bold">Visualization</span>
          <Button
            variantType="secondary"
            variant="ghost"
            size="xs"
            iconComponent="icon-[ic--baseline-delete]"
            onClick={() => {
              if (activeVisualization) removeVisualization();
            }}
          />
        </div>
        <Input
          type="text"
          size="sm"
          value={activeVisualization?.name || ''}
          onChange={(val) => {
            updateSettings({ name: val });
          }}
          label="Name"
          inline
        />
        {activeVisualization && graphMetadata && (
          <>
            <SettingsHeader name="Visualization Settings" />
            <Suspense fallback={<div>Loading...</div>}>
              {component && component.component && activeVisualization && component.id === activeVisualization.id && (
                <component.component settings={activeVisualization} graphMetadata={graphMetadata} updateSettings={updateSettings} />
              )}
            </Suspense>
          </>
        )}
      </div>
    </div>
  );
}
